'use strict';

var React = require('react');
var isEqual = require('@guanghechen/fast-deep-equal');

function _interopDefault (e) { return e && e.__esModule ? e : { default: e }; }

var React__default = /*#__PURE__*/_interopDefault(React);
var isEqual__default = /*#__PURE__*/_interopDefault(isEqual);

function useDeepCompareCallback(fn, deps) {
    const signal = React__default.default.useRef(0);
    const prevDeps = React__default.default.useRef(deps);
    if (!isEqual__default.default(prevDeps.current, deps))
        signal.current += 1;
    prevDeps.current = deps;
    return React__default.default.useCallback(fn, [signal.current]);
}

function useRefreshRef(value) {
    const ref = React__default.default.useRef(value);
    ref.current = value;
    return ref;
}

function useBeforeUnloadEffect(fn, deps) {
    const beforeunload = useDeepCompareCallback(fn, deps);
    React__default.default.useLayoutEffect(() => {
        window.addEventListener('beforeunload', beforeunload);
        return () => window.removeEventListener('beforeunload', beforeunload);
    }, [beforeunload]);
}
function useBeforeUnloadAsyncEffect(fn, deps, preventDefault = true) {
    const preventDefaultRef = useRefreshRef(preventDefault);
    const beforeunload = useDeepCompareCallback(async (event) => {
        if (preventDefaultRef.current === false) {
            return fn(event);
        }
        event.preventDefault();
        try {
            await fn(event);
        }
        finally {
            delete event.returnValue;
        }
    }, deps);
    React__default.default.useLayoutEffect(() => {
        window.addEventListener('beforeunload', beforeunload);
        return () => window.removeEventListener('beforeunload', beforeunload);
    }, [beforeunload]);
}

function useDeepCompareEffect(fn, deps) {
    const signal = React__default.default.useRef(0);
    const prevDeps = React__default.default.useRef(deps);
    if (!isEqual__default.default(prevDeps.current, deps))
        signal.current += 1;
    prevDeps.current = deps;
    React__default.default.useEffect(fn, [signal.current]);
}

function useDeepCompareMemo(fn, deps) {
    const signal = React__default.default.useRef(0);
    const prevDeps = React__default.default.useRef(deps);
    if (!isEqual__default.default(prevDeps.current, deps))
        signal.current += 1;
    prevDeps.current = deps;
    return React__default.default.useMemo(fn, [signal.current]);
}

function useEventCallback(handler) {
    const handlerRef = React__default.default.useRef(handler);
    React__default.default.useLayoutEffect(() => {
        handlerRef.current = handler;
    });
    return React__default.default.useCallback((...args) => {
        const handle = handlerRef.current;
        return handle(...args);
    }, []);
}

function useInterval(callback, duration) {
    const callbackRef = React__default.default.useRef(callback);
    React__default.default.useEffect(() => {
        callbackRef.current = callback;
    }, [callback]);
    React__default.default.useEffect(() => {
        const tick = () => {
            if (callbackRef.current === undefined)
                return;
            callbackRef.current();
        };
        const intervalId = setInterval(tick, duration);
        return () => clearInterval(intervalId);
    }, [duration]);
}

function usePreviousState(value) {
    const ref = React__default.default.useRef(value);
    React__default.default.useEffect(() => {
        ref.current = value;
    }, [value]);
    return ref.current;
}

function useSyncState(initialState) {
    const [state, setState] = React__default.default.useState(initialState);
    React__default.default.useEffect(() => setState(initialState), [initialState]);
    return [state, setState];
}

exports.useBeforeUnloadAsyncEffect = useBeforeUnloadAsyncEffect;
exports.useBeforeUnloadEffect = useBeforeUnloadEffect;
exports.useDeepCompareCallback = useDeepCompareCallback;
exports.useDeepCompareEffect = useDeepCompareEffect;
exports.useDeepCompareMemo = useDeepCompareMemo;
exports.useEventCallback = useEventCallback;
exports.useInterval = useInterval;
exports.usePreviousState = usePreviousState;
exports.useRefreshRef = useRefreshRef;
exports.useSyncState = useSyncState;
